#!/usr/bin/perl
#
# cdedit.perl
#
# this program draws the forms associated with editing tags when
# encoding new CDs into the database.
#

$tunebot_root=$ENV{TUNEBOT_ROOT};
$debug=0;
$ses_directory=$tunebot_root."/data/loader";
$uprefs_directory=$tunebot_root."/data/userprefs";

sub urldec {
    my ($a) = $_[0];
    my ($b) = "";
    $a =~ tr/+/ /;
    while ($a ne "") {
        my ($c) = substr($a, 0, 1);
        if ($c eq "%") {
            $b .= chr hex substr($a, 1, 2);
            $a =~ s/...//;
        } else {
            $b .= $c;
            $a =~ s/.//;
        }
    }
    return $b;
}

sub urlenc {
    my ($a)=$_[0];
    $a =~ s/&/&amp;/g;
    $a =~ s/</&lt;/g;
    $a =~ s/>/&gt;/g;
    return $a;
}

#
# add/replace a tag value into the current session.
# assumes $ses_directory, $usrnam and $sesno are set.
# creates the session if it doesn't already exist.
#
sub addSesTag {
    open (O, ">>$ses_directory/$usrnam.$sesno");
    print O @_, "\n";
    close(O);
    chmod 0666, "$ses_directory/$usrnam.$sesno";
}

#
# lookup a tag value from the current session.
# assumes $ses_directory, $usrnam and $sesno are set.
# returns the tag value, or "" if not defined.
#
sub getSesTag {
    my ($tag) = $_[0] . "=";
    my ($l) = length($tag);
    my $val;
    my $i;
    open (I, "$ses_directory/$usrnam.$sesno");
    while ($i = <I>) {
	chomp $i;
	if (substr($i, 0, $l) eq $tag) {
	    $val = $i;
	    $val =~ s/^[^=]*=//;
	}
    }
    close(I);
    return $val;
}

#
# lookup the entire history of a tag value from the current session.
# assumes $ses_directory, $usrnam and $sesno are set.
# returns an array of values, in order found.
#
sub getSesTagAll {
    my ($tag) = $_[0] . "=";
    my ($l) = length($tag);
    my $val;
    my @lst;
    my $i;
    open (I, "$ses_directory/$usrnam.$sesno");
    while ($i = <I>) {
	chomp $i;
	if (substr($i, 0, $l) eq $tag) {
	    $val = $i;
	    $val =~ s/^[^=]*=//;
	    @lst = (@lst, $val);
	}
    }
    close(I);
    return @lst;
}

#
# reinitialize the session edit buffer.
#
sub initSesEditTag {
    open (O, ">$ses_directory/$usrnam.$sesno"."_e");
    close(O);
    chmod 0666, "$ses_directory/$usrnam.$sesno"."_e";
}

#
# add/replace a tag value into the current session edit buffer.
# assumes $ses_directory, $usrnam and $sesno are set.
# creates the session if it doesn't already exist.
#
sub addSesEditTag {
    open (O, ">>$ses_directory/$usrnam.$sesno"."_e");
    print O @_, "\n";
    close(O);
    chmod 0666, "$ses_directory/$usrnam.$sesno"."_e";
}

#
# lookup a tag value from the current session edit buffer.
# assumes $ses_directory, $usrnam and $sesno are set.
# returns the tag value, or "" if not defined.
#
sub getSesEditTag {
    my ($tag) = $_[0] . "=";
    my ($l) = length($tag);
    my $val;
    my $i;
    open (I, "$ses_directory/$usrnam.$sesno"."_e");
    while ($i = <I>) {
	chomp $i;
	if (substr($i, 0, $l) eq $tag) {
	    $val = $i;
	    $val =~ s/^[^=]*=//;
	}
    }
    close(I);
    return $val;
}

#
# add/replace a tag value into the user preferences
# assumes $uprefs_directory and $usrnam are set.
# creates the preferences file if it doesn't already exist.
#
sub addUPrefsTag {
    open (O, ">>$uprefs_directory/$usrnam");
    print O @_, "\n";
    close(O);
    chmod 0666, "$uprefs_directory/$usrnam";
}

#
# lookup a tag value from the user preferences.
# assumes $uprefs_directory and $usrnam are set.
# returns the tag value, or "" if not defined.
#
sub getUPrefsTag {
    my ($tag) = $_[0] . "=";
    my ($l) = length($tag);
    my ($val) = $_[1];
    my $i;
    open (I, "$uprefs_directory/$usrnam");
    while ($i = <I>) {
	chomp $i;
	if (substr($i, 0, $l) eq $tag) {
	    $val = $i;
	    $val =~ s/^[^=]*=//;
	}
    }
    close(I);
    return $val;
}

if ($debug > 0) {
    print "<p>\n";
    print "query string is: " . $ENV{QUERY_STRING} . "\n";
    print "<p>\n";
    print "i've got " . $#ARGV . " command args\n";
    for ($i = 0; $i <= $#ARGV; $i++) {
	print "<br>" . $i . ". " . @ARGV[$i] . "\n";
    }
    print "<p>\n";
    print "<pre>\n";
}

#
# parse stdin into a list of name-value pairs and stuff into hash %posts.
# this is used to support the POST method.
# stdin format is "name1=value1&name2=value2&...".
#
while ($line = <>) {
    chop $line;
    @nvpairs = split('&', $line);
    for ($i = 0; $i <= $#nvpairs; $i++) {
	@nv = split('=',@nvpairs[$i]);
	if ($#nv == 1) {
	    if ($debug > 0) {
		print @nv[0] . "  =  " . @nv[1] . "\n";
	    }
	    $posts{@nv[0]} = urldec(@nv[1]);
	}
    }
};

#
# parse QUERY_STRING into nv pairs and hash them.
# this supports the GET method (which is currently used).
#
@nvpairs = split('&', $ENV{QUERY_STRING});
for ($i = 0; $i <= $#nvpairs; $i++) {
    @nv = split('=',@nvpairs[$i]);
    if ($#nv == 1) {
	if ($debug > 0) {
	    print @nv[0] . "  =  " . @nv[1] . "\n";
	}
	$posts{@nv[0]} = urldec(@nv[1]);
    }
}
if ($debug > 0) {
    print "</pre>\n";
}

#
# security hole - we trust these as filenames and stuff.
#
$usrnam=$posts{"usrnam"};
$usrnam =~ tr/[A-Za-z0-9]//dc;
$sesno=$posts{"sesno"};
$sesno =~ tr/[0-9]//dc;

#
# special action - begin by copying session data to temp
#
if ($usrnam ne "" && $sesno ne "" && $posts{"begin"} ne "") {
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdedit.cgi?usrnam=$usrnam&sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    initSesEditTag();
    addSesEditTag("ALBUMDATE=".getSesTag("ALBUMDATE"));
    addSesEditTag("ALBUMDIRECTORY=".getSesTag("ALBUMDIRECTORY"));
    addSesEditTag("ALBUMNAME=".getSesTag("ALBUMNAME"));
    addSesEditTag("ARTISTDIRECTORY=".getSesTag("ARTISTDIRECTORY"));
    addSesEditTag("ARTISTNAME=".getSesTag("ARTISTNAME"));
    addSesEditTag("ARTISTSORT=".getSesTag("ARTISTSORT"));
    addSesEditTag("ENCODINGPLAN=".getSesTag("ENCODINGPLAN"));
    addSesEditTag("MP3RATE=".getSesTag("MP3RATE"));
    addSesEditTag("NTRACKS=".getSesTag("NTRACKS"));
    $ntracks = getSesTag("NTRACKS");
    for ($i = 1; $i <= $ntracks; ++$i) {
	addSesEditTag("TRACKARTIST$i=".getSesTag("TRACKARTIST$i"));
	addSesEditTag("TRACKNAME$i=".getSesTag("TRACKNAME$i"));
    }
    exit 0;
}

#
# special action - enable session debugging
#
if ($usrnam ne "" && $sesno ne "" && $posts{"debugses"} ne "") {
    addSesTag("SESSIONDEBUG=".$posts{"debugses"});
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdedit.cgi?usrnam=$usrnam&sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    exit 0;
}

#
# special action - return to session display with no changes
#
if ($posts{"cancel"} ne "") {
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdload.cgi?usrnam=$usrnam&sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    exit 0;
}

#
# special action - check in the current form contents and redisplay
#
if ($usrnam ne "" && $sesno ne "" && $posts{"check"} ne "") {
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdedit.cgi?usrnam=$usrnam&sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    addSesEditTag("ALBUMDATE=".$posts{"albumdate"});
    addSesEditTag("ALBUMDATE=".$posts{"albumdate"});
    addSesEditTag("ALBUMDIRECTORY=".$posts{"albumdirectory"});
    addSesEditTag("ALBUMNAME=".$posts{"albumname"});
    addSesEditTag("ARTISTDIRECTORY=".$posts{"artistdirectory"});
    addSesEditTag("ARTISTNAME=".$posts{"artistname"});
    addSesEditTag("ARTISTSORT=".$posts{"artistsort"});
    addSesEditTag("ENCODINGPLAN=".$posts{"encodingplan"});
    if ($posts{"mp3rate"} == 128) {
	addSesEditTag("MP3RATE=128");
    }
    if ($posts{"mp3rate"} == 192) {
	addSesEditTag("MP3RATE=192");
    }
    if ($posts{"mp3rate"} == 256) {
	addSesEditTag("MP3RATE=256");
    }
    $ntracks = getSesTag("NTRACKS");
    for ($i = 1; $i <= $ntracks; ++$i) {
	addSesEditTag("TRACKARTIST$i=".$posts{"trackartist$i"});
	addSesEditTag("TRACKNAME$i=".$posts{"trackname$i"});
    }
    exit 0;
}

#
# special action - swap track titles and artists
#
if ($usrnam ne "" && $sesno ne "" && $posts{"swap"} ne "") {
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdedit.cgi?usrnam=$usrnam&sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    $ntracks = getSesEditTag("NTRACKS");
    for ($i = 1;  $i <= $ntracks; ++$i) {
	$t = getSesEditTag("TRACKARTIST$i");
	addSesEditTag("TRACKARTIST$i=".getSesEditTag("TRACKNAME$i"));
	addSesEditTag("TRACKNAME$i=$t");
    }
    exit 0;
}

#
# special action - check in the current form contents and redisplay
#
if ($usrnam ne "" && $sesno ne "" && $posts{"submit"} ne "") {
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdload.cgi?usrnam=$usrnam&sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    addSesTag("ALBUMDATE=".getSesEditTag("ALBUMDATE"));
    addSesTag("ALBUMDIRECTORY=".getSesEditTag("ALBUMDIRECTORY"));
    addSesTag("ALBUMNAME=".getSesEditTag("ALBUMNAME"));
    addSesTag("ARTISTDIRECTORY=".getSesEditTag("ARTISTDIRECTORY"));
    addSesTag("ARTISTNAME=".getSesEditTag("ARTISTNAME"));
    addSesTag("ARTISTSORT=".getSesEditTag("ARTISTSORT"));
    addSesTag("ENCODINGPLAN=".getSesEditTag("ENCODINGPLAN"));
    addSesTag("MP3RATE=".getSesEditTag("MP3RATE"));
    addSesTag("NTRACKS=".getSesEditTag("NTRACKS"));
    $ntracks = getSesTag("NTRACKS");
    for ($i = 1; $i <= $ntracks; ++$i) {
	addSesTag("TRACKARTIST$i=".getSesEditTag("TRACKARTIST$i"));
	addSesTag("TRACKNAME$i=".getSesEditTag("TRACKNAME$i"));
    }
    exit 0;
}

#
# normal (no action) - begin to render the session status display
#
print "Content-type: text/html\n";
print "\n";
print "<html>\n";
print "<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n";
print "<head>\n";
print "<title>" . $ENV{BANNER_STRING} . " RipBotEdit</title>\n";
print "</head>\n";
open(I, $tunebot_root . "/html/bodypart.html.head");
while ($line = <I>) {
    print $line;
}
close(I);

#
# print RipBot banner and user+session if they're defined.
#
print "<p>\n";
print "<center><font size=+2>" . $ENV{BANNER_STRING} . " RipBotEdit</font>";
if ($usrnam ne "") {
    print " <font size=+2 color=\"#723a4c\"> [ $usrnam";
    if ($sesno ne "") {
	print "#$sesno";
	print "<em> / ".getSesTag("NAME")."</em> : ".getSesTag("STATE");
    }
    print " ] </font>\n";
}
print "</center>\n";
print "<p>\n";

#
# now we render whichever form is appropriate for the current
# state of the user / session and edit.
#

SWITCH: {

#
# this is for when there's no username.  only option is to log in.
#
    if ($usrnam eq "" || $sesno eq "") {
	print "Internal Error.  I can't find your session\n";
	last SWITCH;
    }

#
# now we have a user and session number
# render the appropriate session form
#
    if ($posts{"edittags"} eq "") {
	print "<form method=post action=\"/cgi-bin/cdedit.cgi\">";
	print "<input type=hidden name=usrnam value=\"$usrnam\">\n";
	print "<input type=hidden name=sesno value=\"$sesno\">\n";
	print "<table cellpadding=0 cellspacing=0>\n";
	print "<tr><td colspan=3>\n";
	print "<input type=submit name=cancel value=\"Discard Changes\">\n";
	print "<input type=submit name=submit value=\"Save Changes\">\n";
	print "<input type=submit name=check value=\"Check Changes\">\n";
	print "<input type=submit name=split value=\"Split Track Titles\">\n";
	print "<input type=submit name=swap value=\"Swap Track and Artist\">\n";
	print "</td></tr>\n";
	print "<tr><td><em>Artist Name:</em></td>";
	print "<td><input name=artistname value=\"".urlenc(getSesEditTag("ARTISTNAME"))."\"></td>\n";
	print "<td><font size=-1><em>The artist's name as read by humans</em></font></td></tr>\n";
	print "<tr><td><em>Artist Sort Key:</em></td>";
	print "<td><input name=artistsort value=\"".urlenc(getSesEditTag("ARTISTSORT"))."\"></td>\n";
	print "<td><font size=-1><em>The artist's name as you would sort it in a list</em></font></td></tr>\n";
	print "<tr><td><em>Directory:</em></td>";
	print "<td><input name=artistdirectory value=\"".urlenc(getSesEditTag("ARTISTDIRECTORY"))."\"></td>\n";
	print "<td><font size=-1><em>The directory in which to store the album data.  By convention, this should be all one word, in lowercase, with underscores.</em></font></td></tr>\n";
	print "<tr><td><em>Album Name:</em></td>";
	print "<td><input name=albumname value=\"".urlenc(getSesEditTag("ALBUMNAME"))."\"></td>\n";
	print "<td><font size=-1><em>The album name as read by humans</em></font></td></tr>\n";
	print "<tr><td><em>Directory:</em></td>";
	print "<td><input name=albumdirectory value=\"".urlenc(getSesEditTag("ALBUMDIRECTORY"))."\"></td>\n";
	print "<td><font size=-1><em>Directory in which to store album.  By convention, this should be all one word, in lowercase, with underscores.</em></font></td></tr>\n";
	print "<tr><td><em>Release Date:</em></td>";
	print "<td><input name=albumdate value=\"".urlenc(getSesEditTag("ALBUMDATE"))."\"></td>\n";
	print "<td><font size=-1><em>The year the album was released.</em></font></td></tr>\n";
	print "<tr><td><em>CDDB:</em></td><td> ".getSesTag("CDDB")." </td></tr>\n";
	print "<tr><td><em>Encoding Plan:</em></td><td> ".getSesEditTag("ENCODINGPLAN")." </td></tr>\n";
	print "<tr><td><em>MP3 Bitrate:</em></td><td> ";
	$x = getSesEditTag("MP3RATE");
	if ($x == 128) {
	    print "<input type=radio name=mp3rate value=128 checked> 128 ";
	} else {
	    print "<input type=radio name=mp3rate value=128> 128 ";
	}
	if ($x == 192) {
	    print "<input type=radio name=mp3rate value=192 checked> 192 ";
	} else {
	    print "<input type=radio name=mp3rate value=192> 192 ";
	}
	if ($x == 256) {
	    print "<input type=radio name=mp3rate value=256 checked> 256 ";
	} else {
	    print "<input type=radio name=mp3rate value=256> 256 ";
	}
	print "</td>\n";
	print "<td><font size=-1><em>The MP3 bitrate (kbit/s) for encoding.</em></font></td></tr>\n";
	$x = (getSesTag("TOTLENGTH") * (getSesEditTag("MP3RATE") * 1024 / 8)) / 1048576;
	$x = int($x + 0.5);
	print "<tr><td><em>Est'd Disk Space:</em></td><td> $x MB </td></tr>\n";
	$ntracks = getSesEditTag("NTRACKS");
	print "<tr><td><em>Tracks:</em></td><td> $ntracks </td></tr>\n";
	print "</table>\n";
	print "<table width=80% cellpadding=0 cellspacing=0>\n";
	print "<tr><th><em>Title</em></th><th><em>Artist (optional)</em></th><th><em>Time</em></th></tr>\n";
	print "<tr><td colspan=3><hr></td></tr>\n";
	for ($i = 1; $i <= $ntracks; ++$i) {
#	    print "<tr><td width=50% align=left><font size=-1> $i. </font>".getSesEditTag("TRACKNAME$i")." </td>";
	    print "<tr><td width=50% align=left><font size=-1> $i. </font>";
	    print "<input name=trackname$i value=\"".urlenc(getSesEditTag("TRACKNAME$i"))."\"></td>\n";
	    print "<td width=40%>";
	    print "<input name=trackartist$i value=\"".urlenc(getSesEditTag("TRACKARTIST$i"))."\"></td>\n";
	    print "<td> ".getSesTag("TRACKTIME$i")." </td></tr>\n";
	}
	print "<tr><td colspan=3><hr></td></tr>\n";
	print "</table>\n";
	print "</form>\n";
	last SWITCH;
    }
}

#
# this is for debugging.
# it lists all the session tags at the end of the form.
# XXX this could be switched on a session DEBUG tag
#
if (getSesTag("SESSIONDEBUG") == 1) {
    print "<p>\n";
    print "<center>*</center>\n";
    print "<hr>\n";
    print "<p>\n";
    print "<center>Session Tags</center>\n";
    print "<font color=white>\n";
    print "<ul><pre>\n";
    open(I, "$ses_directory/$usrnam.$sesno");
    while (<I>) {
	print;
    }
    close(I);
    print "</pre></ul>\n";
    print "</font>\n";
    print "<hr>\n";
    print "<font size=-1><em>Disable</em> <a href=\"/cgi-bin/cdedit.cgi?usrnam=$usrnam&sesno=$sesno&debugses=0\">Session Debugging</a></font>\n";
} else {
    if ($sesno ne "") {
	print "<p>\n";
	print "<font size=-1><em>Enable</em> <a href=\"/cgi-bin/cdedit.cgi?usrnam=$usrnam&sesno=$sesno&debugses=1\">Session Debugging</a></font>\n";
    }
}

open(I, $tunebot_root . "/html/bodypart.html.tail");
while ($line = <I>) {
    print $line;
}
close(I);
print "</html>\n";

exit 0

# ________________________________________________________________________

    #	    print "<table><tr><td>";
    #	    print "<form action=\"/cgi-bin/cdload.cgi\">";
    #	    print "<input type=hidden name=usrnam value=\"$usrnam\">\n";
    #	    print "<input type=hidden name=sesno value=\"$sesno\">\n";
    #	    print "<input type=submit name=splitleft value=\"Split - Artist on Left\">";
    #	    print "<input type=submit name=splitright value=\"Split - Artist on Right\">";
    #	    print "</td></tr></table>\n";
    #	print "<form action=\"/cgi-bin/cdload.cgi\">\n";
    #	print "<input type=hidden name=usrnam value=\"$usrnam\">\n";
    #	print "<input type=hidden name=sesno value=\"$sesno\">\n";
    #	print "<input type=image name=delete src=\"/rfm/cleanup.gif\" border=0>";
    #	print " - Delete Session";
    #	print "</form>\n";
    #	print "<form action=\"/cgi-bin/cdload.cgi\">\n";
    #	print "<input type=hidden name=usrnam value=\"$usrnam\">\n";
    #	print "<input type=image name=seslist src=\"/rfm/burst.gif\" border=0>";
    #	print " - Back to Session List";
    #	print "</form>\n";
